/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.ui.properties;

import static org.eclipse.jface.databinding.swt.SWTObservables.observeSelection;
import static org.eclipse.jface.databinding.swt.SWTObservables.observeText;
import static org.fortiss.af3.expression.ui.databinding.validate.IntToStringConverter.INT_STRING_CONVERTER;
import static org.fortiss.af3.expression.ui.databinding.validate.StringToIntConverter.STRING_INT_CONVERTER;
import static org.fortiss.tooling.kernel.ui.util.DataBindingUtils.performComplexTextBinding;
import static org.fortiss.tooling.kernel.ui.util.ObservableUtils.observeValue;

import org.conqat.ide.commons.ui.databinding.validate.TextToIntegerValidator;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.fortiss.af3.platform.raspberry.model.RaspberryPi;
import org.fortiss.af3.platform.raspberry.model.RaspberryPiModelPackage;
import org.fortiss.tooling.kernel.ui.extension.base.PropertySectionBase;

/** {@link PropertySectionBase} for {@link RaspberryPi}s. */
public class RaspberryPiExecutionUnitPropertySection extends PropertySectionBase {
	/** The execution unit. */
	private RaspberryPi unit;
	/** The CAN ID of this unit. */
	private Text canCoordinationIDText;
	/** The cycle time widget. */
	private Text cycleTimeText;
	/** The IP address widget. */
	private Text ipAddressText;
	/** The coordinator flag button. */
	private Button coordinatorFlagButton;
	/** The camera server IP address widget. */
	private Text cameraServerAddressText;
	/** The camera server IP port widget. */
	private Text cameraServerPortText;

	/** {@inheritDoc} */
	@Override
	public void createControls(Composite parent, TabbedPropertySheetPage tabbedPropertySheetPage) {
		super.createControls(parent, tabbedPropertySheetPage);
		canCoordinationIDText = createDecoratedText("CAN Unit ID");
		cycleTimeText = createDecoratedText("Cycle Time (ms)");
		coordinatorFlagButton = getWidgetFactory().createButton(composite, "", SWT.CHECK);
		createFormEntry(coordinatorFlagButton, "Coordinator Unit");
		coordinatorFlagButton.setSelection(false);
		ipAddressText = createDecoratedText("Unit IP Address");
		cameraServerAddressText = createDecoratedText("Camera Server IP Address");
		cameraServerPortText = createDecoratedText("Camera Server IP Port");
	}

	/** {@inheritDoc} */
	@Override
	protected void setSectionInput(Object input) {
		unit = (RaspberryPi)input;
	}

	/** {@inheritDoc} */
	@Override
	public void refresh() {
		super.refresh();

		IObservableValue modelObservable =
				observeValue(unit,
						RaspberryPiModelPackage.Literals.RASPBERRY_PI__CAN_COORDINATION_ID);
		performComplexTextBinding(dbc, canCoordinationIDText, modelObservable,
				INT_STRING_CONVERTER, STRING_INT_CONVERTER, TextToIntegerValidator.INSTANCE,
				new BoundedIntPositiveZeroValidator(245, 255));

		modelObservable =
				observeValue(unit, RaspberryPiModelPackage.Literals.RASPBERRY_PI__CYCLE_TIME);
		performComplexTextBinding(dbc, cycleTimeText, modelObservable, INT_STRING_CONVERTER,
				STRING_INT_CONVERTER, TextToIntegerValidator.INSTANCE,
				new BoundedIntPositiveZeroValidator(20, 10000));

		modelObservable =
				observeValue(unit, RaspberryPiModelPackage.Literals.RASPBERRY_PI__IP_ADDRESS);
		dbc.bindValue(observeText(ipAddressText, SWT.Modify), modelObservable);

		modelObservable =
				observeValue(unit, RaspberryPiModelPackage.Literals.RASPBERRY_PI__COORDINATOR_UNIT);
		dbc.bindValue(observeSelection(coordinatorFlagButton), modelObservable);

		modelObservable =
				observeValue(unit,
						RaspberryPiModelPackage.Literals.RASPBERRY_PI__CAMERA_SERVER_ADDRESS);
		dbc.bindValue(observeText(cameraServerAddressText, SWT.Modify), modelObservable);

		modelObservable =
				observeValue(unit,
						RaspberryPiModelPackage.Literals.RASPBERRY_PI__CAMERA_SERVER_PORT);
		dbc.bindValue(observeText(cameraServerPortText, SWT.Modify), modelObservable);
	}
}
